==> cosmic-lib-1.7.0.tgz: package/src/parse-sep7.js <==
"use strict"
/**
 * Sep-0007 support
 * https://github.com/stellar/stellar-protocol/blob/master/ecosystem/sep-0007.md
 *
 * @private
 */
const parseSep7 = exports

const Buffer = require("@cosmic-plus/base/es5/buffer")
const StellarSdk = require("@cosmic-plus/base/es5/stellar-sdk")
const { timeout } = require("@cosmic-plus/jsutils/es5/misc")

const check = require("./check")
const convert = require("./convert")
const decode = require("./decode")
const parse = require("./parse")

/**
 * Initialize cosmicLink using `sep7Request`.
 */
parseSep7.request = function (cosmicLink, sep7Request, options) {
  parse.page(cosmicLink, sep7Request)

  const query = convert.uriToQuery(cosmicLink, sep7Request)
  const sep7 = decodeURIComponent(query.substr(5))
  return parseSep7.link(cosmicLink, sep7, options)
}

/**
 * Initialize cosmicLink using `sep7`.
 */
parseSep7.link = function (cosmicLink, sep7, options = {}) {
  cosmicLink._sep7 = sep7
  if (!options.network) options.network = "public"
  if (sep7.substr(12, 4) === "pay?") {
    cosmicLink.extra.type = "pay"
    return parseSep7.link.pay(cosmicLink, sep7, options)
  } else if (sep7.substr(12, 3) === "tx?") {
    cosmicLink.extra.type = "tx"
    return parseSep7.link.xdr(cosmicLink, sep7, options)
  } else {
    throw new Error("Invalid SEP-0007 link.")
  }
}

/**
 * Initialize cosmicLink using `sep7.xdr`.
 */
parseSep7.link.xdr = function (cosmicLink, sep7, options = {}) {
  const query = convert.uriToQuery(cosmicLink, sep7)
  const params = query.substr(1).split("&")
  let xdr

  params.forEach(entry => {
    const field = entry.replace(/=.*$/, "")
    const value = entry.substr(field.length + 1)

    switch (field) {
    case "xdr":
      xdr = decodeURIComponent(value)
      break
    case "pubkey":
      check.address(cosmicLink, value)
      cosmicLink.extra.pubkey = value
      break
    default:
      parseSep7.link.common(cosmicLink, "xdr", field, value, options)
    }
  })

  if (!xdr) throw new Error("Missing XDR parameter")
  return { type: "xdr", value: xdr, options }
}

/**
 * Initialize cosmicLink using `sep7.pay`.
 */
parseSep7.link.pay = function (cosmicLink, sep7, options = {}) {
  const query = convert.uriToQuery(cosmicLink, sep7)
  const params = query.substr(1).split("&")

  const odesc = { type: "payment" },
    asset = [],
    memo = []
  const tdesc = {
    network: options.network,
    operations: [odesc]
  }

  // Parse
  params.forEach(entry => {
    const field = entry.replace(/=.*$/, "")
    const value = entry.substr(field.length + 1)

    switch (field) {
    case "destination":
    case "amount":
      odesc[field] = decode.field(cosmicLink, field, value)
      break
    case "asset_code":
      asset[0] = value
      break
    case "asset_issuer":
      asset[1] = value
      break
    case "memo_type":
      memo[0] = value.split("_")[1].toLowerCase()
      break
    case "memo":
      memo[1] = value
      break
    default:
      parseSep7.link.common(cosmicLink, "pay", field, value, options)
    }
  })

  // Convert
  if (memo.length) {
    if (memo[0] === "hash" || memo[0] === "return") {
      const buffer = Buffer.from(decodeURIComponent(memo[1]), "base64")
      memo[1] = buffer.toString("hex")
    }
    tdesc.memo = decode.memo(cosmicLink, `${memo[0]}:${memo[1]}`)
  }
  if (asset.length) {
    odesc.asset = decode.asset(cosmicLink, `${asset[0]}:${asset[1]}`)
  }

  if (!odesc.destination) throw new Error("Missing parameter: destination")
  return { type: "tdesc", value: tdesc, options }
}

parseSep7.link.common = function (cosmicLink, mode, field, value, options) {
  switch (field) {
  case "network_passphrase":
    options.network = decode.network(cosmicLink, value)
    break
  case "callback":
    value = decode.url(cosmicLink, value)
    if (value.substr(0, 4) !== "url:") {
      throw new Error("Invalid callback: " + value)
    }
    options.callback = value.substr(4)
    break
  case "origin_domain":
    cosmicLink.extra.originDomain = parseSep7.checkSignature(
      cosmicLink,
      value
    )
    break
  case "signature":
    cosmicLink.extra.signature = decodeURIComponent(value)
    break
  case "msg":
    cosmicLink.extra.msg = decode.string(cosmicLink, value)
    break
  default:
    throw new Error(`Invalid SEP-0007 ${mode} field: ` + field)
  }
}

parseSep7.checkSignature = async function (cosmicLink, domain) {
  const link = cosmicLink.sep7.replace(/&signature=.*/, "")

  // Let parser parse signature.
  await timeout(1)
  const signature = cosmicLink.extra.signature

  if (!signature) {
    throw new Error(`No signature attached for domain: ${domain}`)
  }

  const toml = await StellarSdk.StellarTomlResolver.resolve(domain)
  const signingKey = toml.URI_REQUEST_SIGNING_KEY

  if (!signingKey) {
    throw new Error(`Can't find signing key for domain: ${domain}`)
  }

  const keypair = StellarSdk.Keypair.fromPublicKey(signingKey)
  const payload = makePayload(link)

  if (keypair.verify(payload, Buffer.from(signature, "base64"))) {
    return domain
  } else {
    throw new Error(`Invalid signature for domain: ${domain}`)
  }
}

function makePayload (link) {
  return Buffer.concat([
    Buffer.alloc(35),
    Buffer.alloc(1, 4),
    Buffer.from(`stellar.sep.7 - URI Scheme${link}`)
  ])
}

==> cosmic-lib-1.8.0.tgz: package/src/parse-sep7.js <==
"use strict"
/**
 * Sep-0007 support
 * https://github.com/stellar/stellar-protocol/blob/master/ecosystem/sep-0007.md
 *
 * @private
 */
const parseSep7 = exports

const Buffer = require("@cosmic-plus/base/es5/buffer")
const StellarSdk = require("@cosmic-plus/base/es5/stellar-sdk")
const { timeout } = require("@cosmic-plus/jsutils/es5/misc")

const check = require("./check")
const convert = require("./convert")
const decode = require("./decode")
const parse = require("./parse")

/**
 * Initialize cosmicLink using `sep7Request`.
 */
parseSep7.request = function (cosmicLink, sep7Request, options) {
  parse.page(cosmicLink, sep7Request)

  const query = convert.uriToQuery(cosmicLink, sep7Request)
  const sep7 = decodeURIComponent(query.substr(5))
  return parseSep7.link(cosmicLink, sep7, options)
}

/**
 * Initialize cosmicLink using `sep7`.
 */
parseSep7.link = function (cosmicLink, sep7, options = {}) {
  cosmicLink._sep7 = sep7
  if (!options.network) options.network = "public"
  if (sep7.substr(12, 4) === "pay?") {
    cosmicLink.extra.type = "pay"
    return parseSep7.link.pay(cosmicLink, sep7, options)
  } else if (sep7.substr(12, 3) === "tx?") {
    cosmicLink.extra.type = "tx"
    return parseSep7.link.xdr(cosmicLink, sep7, options)
  } else {
    throw new Error("Invalid SEP-0007 link.")
  }
}

/**
 * Initialize cosmicLink using `sep7.xdr`.
 */
parseSep7.link.xdr = function (cosmicLink, sep7, options = {}) {
  const query = convert.uriToQuery(cosmicLink, sep7)
  const params = query.substr(1).split("&")
  let xdr

  params.forEach(entry => {
    const field = entry.replace(/=.*$/, "")
    const value = entry.substr(field.length + 1)

    switch (field) {
    case "xdr":
      xdr = decodeURIComponent(value)
      break
    case "pubkey":
      check.address(cosmicLink, value)
      cosmicLink.extra.pubkey = value
      break
    default:
      parseSep7.link.common(cosmicLink, "xdr", field, value, options)
    }
  })

  if (!xdr) throw new Error("Missing XDR parameter")
  return { type: "xdr", value: xdr, options }
}

/**
 * Initialize cosmicLink using `sep7.pay`.
 */
parseSep7.link.pay = function (cosmicLink, sep7, options = {}) {
  const query = convert.uriToQuery(cosmicLink, sep7)
  const params = query.substr(1).split("&")

  const odesc = { type: "payment" },
    asset = {},
    memo = {}
  const tdesc = {
    network: options.network,
    operations: [odesc]
  }

  // Parse
  params.forEach(entry => {
    const field = entry.replace(/=.*$/, "")
    const value = entry.substr(field.length + 1)

    switch (field) {
    case "destination":
    case "amount":
      odesc[field] = decode.field(cosmicLink, field, value)
      break
    case "asset_code":
      asset.code = decodeURIComponent(value)
      break
    case "asset_issuer":
      asset.issuer = decodeURIComponent(value)
      break
    case "memo_type":
      memo.type = value.split("_")[1].toLowerCase()
      break
    case "memo":
      memo.value = decode.string(cosmicLink, value)
      break
    default:
      parseSep7.link.common(cosmicLink, "pay", field, value, options)
    }
  })

  // Convert
  if (memo.type || memo.value) {
    if (memo.type === "hash" || memo.type === "return") {
      memo.value = Buffer.from(memo.value, "base64").toString("hex")
    }
    tdesc.memo = memo
  }
  if (asset.code || asset.issuer) odesc.asset = asset

  if (!odesc.destination) throw new Error("Missing parameter: destination")
  return { type: "tdesc", value: tdesc, options }
}

parseSep7.link.common = function (cosmicLink, mode, field, value, options) {
  switch (field) {
  case "network_passphrase":
    options.network = decode.network(cosmicLink, value)
    break
  case "callback":
    value = decode.url(cosmicLink, value)
    if (value.substr(0, 4) !== "url:") {
      throw new Error("Invalid callback: " + value)
    }
    options.callback = value.substr(4)
    break
  case "origin_domain":
    cosmicLink.extra.originDomain = parseSep7.checkSignature(
      cosmicLink,
      value
    )
    break
  case "signature":
    cosmicLink.extra.signature = decodeURIComponent(value)
    break
  case "msg":
    cosmicLink.extra.msg = decode.string(cosmicLink, value)
    break
  default:
    throw new Error(`Invalid SEP-0007 ${mode} field: ` + field)
  }
}

parseSep7.checkSignature = async function (cosmicLink, domain) {
  const link = cosmicLink.sep7.replace(/&signature=.*/, "")

  // Let parser parse signature.
  await timeout(1)
  const signature = cosmicLink.extra.signature

  if (!signature) {
    throw new Error(`No signature attached for domain: ${domain}`)
  }

  const toml = await StellarSdk.StellarTomlResolver.resolve(domain)
  const signingKey = toml.URI_REQUEST_SIGNING_KEY

  if (!signingKey) {
    throw new Error(`Can't find signing key for domain: ${domain}`)
  }

  const keypair = StellarSdk.Keypair.fromPublicKey(signingKey)
  const payload = makePayload(link)

  if (keypair.verify(payload, Buffer.from(signature, "base64"))) {
    return domain
  } else {
    throw new Error(`Invalid signature for domain: ${domain}`)
  }
}

function makePayload (link) {
  return Buffer.concat([
    Buffer.alloc(35),
    Buffer.alloc(1, 4),
    Buffer.from(`stellar.sep.7 - URI Scheme${link}`)
  ])
}
